/* johd: Main Jabber-Over-HTTP daemon.
   Copyright (C) 2011 Sergey Poznyakoff

   JOH is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 3, or (at your option)
   any later version.

   JOH is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with JOH.  If not, see <http://www.gnu.org/licenses/>. */

#include "joh.h"
#include <unistd.h>

struct joh_sockaddr *listen_addrs;
struct joh_sockaddr *jabber_addr;

void
version()
{
	printf("%s (%s) %s\n", program_name, PACKAGE, PACKAGE_VERSION);
	printf("Copyright (C) 2011 Sergey Poznyakoff");
	puts("\
\n\
License GPLv3+: GNU GPL version 3 or later <http://gnu.org/licenses/gpl.html>\n\
This is free software: you are free to change and redistribute it.\n\
There is NO WARRANTY, to the extent permitted by law.\n\
\n\
");
	exit(EX_OK);
}

void
help()
{
	printf("Usage: johd [OPTIONS]\n");
	printf("johd provides a jabber-over-HTTP tunnel (XEP-0025)\n");
	printf("OPTIONS:\n");
	puts("\
  -d NUMBER        set debugging level\n\
  -i               show source line information with debug messages\n\
  -l URL           listen on the given IP and PORT (several -l options are OK)\n\
  -s URL           jabber server\n\
\n\
  -V               print the program version\n\
  -h               show this help summary\n\
\n");
	printf("Report bugs to <%s>.\n", PACKAGE_BUGREPORT);
	exit(EX_OK);
}

int
main(int argc, char **argv)
{
	int c;
	char *p;
	static struct joh_sockaddr_hints
		listen_hints = { JOH_HINT_BIND|JOH_HINT_SERVICE,
				 DEFAULT_PORT_STRING },
		jabber_hints = { JOH_HINT_SERVICE, JABBER_PORT_STRING };

	set_program_name(argv[0]);
	while ((c = getopt(argc, argv, "d:il:s:hV")) != EOF) {
		switch (c) {
		case 'd':
			debug_level = strtoul(optarg, &p, 10);
			if (*p)
				die(EX_USAGE,
				    "debug level must be a number");
			else if (debug_level < 0 || debug_level > 100)
				die(EX_USAGE,
				    "debug level out of range 0..100");
			break;

		case 'i':
			source_info_option = 1;
			break;
			
		case 'l':
			listen_addrs =
				joh_sockaddr_append(listen_addrs,
						parse_sockaddr(optarg,
							       &listen_hints));
			break;
				
		case 's':
			if (jabber_addr)
				die(EX_USAGE,
				    "only one -s option is allowed");
			jabber_addr = parse_sockaddr(optarg,
						     &jabber_hints);
			if (jabber_addr->next) {
				warn("%s resolves to more than one address",
				     optarg);
				joh_sockaddr_free(jabber_addr->next);
				jabber_addr->next = NULL;
			}
			break;
			
		case 'h':
			help();
			break;

		case 'V':
			version();
			break;

		default:
			exit(EX_USAGE);
		}
	}

	if (!listen_addrs)
		listen_addrs = parse_sockaddr("inet://", &listen_hints);
	if (!jabber_addr) {
		/* FIXME: Use SRV record, if available. Ditto for -l. */
		jabber_addr = parse_sockaddr(JABBER_SERVER, &jabber_hints);
		if (jabber_addr->next) {
			joh_sockaddr_free(jabber_addr->next);
			jabber_addr->next = NULL;
		}
	}

	joh_conntab_init();
	joh_listen_socket_setup(listen_addrs);
	joh_loop();
	joh_listen_socket_cleanup();
	
	return EX_OK;
}
